-- SPATIAL_INDEX.sql
--
-- Authors:     Prof. Dr. Thomas H. Kolbe <thomas.kolbe@tum.de>
--              Zhihang Yao <zhihang.yao@tum.de>
--              Claus Nagel <cnagel@virtualcitysystems.de>
--              Philipp Willkomm <pwillkomm@moss.de>
--              Gerhard Knig <gerhard.koenig@tu-berlin.de>
--              Alexandra Lorenz <di.alex.lorenz@googlemail.com>
--
-- Copyright:   (c) 2012-2014  Chair of Geoinformatics,
--                             Technische Universitt Mnchen, Germany
--                             http://www.gis.bv.tum.de
--
--              (c) 2007-2012  Institute for Geodesy and Geoinformation Science,
--                             Technische Universitt Berlin, Germany
--                             http://www.igg.tu-berlin.de
--
--              This skript is free software under the LGPL Version 2.1.
--              See the GNU Lesser General Public License at
--              http://www.gnu.org/copyleft/lgpl.html
--              for more details.
-------------------------------------------------------------------------------
-- About:
--
--
-------------------------------------------------------------------------------
--
-- ChangeLog:
--
-- Version | Date       | Description                               | Author
-- 3.0.0     2013-12-06   new version for 3DCityDB V3                 ZYao
--                                                                    TKol
--                                                                    CNag
--                                                                    PWil
-- 2.0.0     2007-11-23   release version                             TKol
--                                                                    GKoe
--                                                                    CNag
--                                                                    ALor
--
--// ENTRIES INTO USER_SDO_GEOM_METADATA WITH BOUNDING VOLUME WITH 10000km EXTENT FOR X,Y AND 11km FOR Z

SET SERVEROUTPUT ON
SET FEEDBACK ON

ALTER SESSION set NLS_TERRITORY='AMERICA';
ALTER SESSION set NLS_LANGUAGE='AMERICAN';

-- Fetch SRID from the the table DATABASE_SRS

VARIABLE SRID NUMBER;
BEGIN
  SELECT SRID INTO :SRID FROM DATABASE_SRS;
END;
/

-- Transfer the value from the bind variable :SRID to the substitution variable &SRSNO
column mc new_value SRSNO print
select :SRID mc from dual;

prompt Used SRID for spatial indexes: &SRSNO 


DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BREAKLINE_RELIEF' AND COLUMN_NAME='BREAK_LINES';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BREAKLINE_RELIEF','BREAK_LINES',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BREAKLINE_RELIEF' AND COLUMN_NAME='RIDGE_OR_VALLEY_LINES';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BREAKLINE_RELIEF','RIDGE_OR_VALLEY_LINES',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD1_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD1_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD2_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD2_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD2_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD2_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD3_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD3_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD3_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD3_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD4_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD4_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE' AND COLUMN_NAME='LOD4_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE','LOD4_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD1_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD1_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD2_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD2_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD3_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD3_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD4_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD4_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD1_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD1_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD1_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD1_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_CONSTR_ELEMENT' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_CONSTR_ELEMENT','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_FURNITURE' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_FURNITURE','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_FURNITURE' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_FURNITURE','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_INSTALLATION' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_INSTALLATION','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_INSTALLATION' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_INSTALLATION','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_INSTALLATION' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_INSTALLATION','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_INSTALLATION' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_INSTALLATION','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_INSTALLATION' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_INSTALLATION','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_INSTALLATION' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_INSTALLATION','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_OPENING' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_OPENING','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BRIDGE_OPENING' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BRIDGE_OPENING','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD1_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD1_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD2_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD2_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD2_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD2_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD3_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD3_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD3_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD3_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD4_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD4_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING' AND COLUMN_NAME='LOD4_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING','LOD4_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_FURNITURE' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_FURNITURE','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_FURNITURE' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_FURNITURE','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_INSTALLATION' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_INSTALLATION','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_INSTALLATION' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_INSTALLATION','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_INSTALLATION' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_INSTALLATION','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_INSTALLATION' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_INSTALLATION','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_INSTALLATION' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_INSTALLATION','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='BUILDING_INSTALLATION' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('BUILDING_INSTALLATION','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITYMODEL' AND COLUMN_NAME='ENVELOPE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITYMODEL','ENVELOPE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITYOBJECT' AND COLUMN_NAME='ENVELOPE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITYOBJECT','ENVELOPE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITYOBJECTGROUP' AND COLUMN_NAME='OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITYOBJECTGROUP','OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD1_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD1_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD1_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD1_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD1_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD1_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD2_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD2_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD3_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD3_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD4_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD4_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='CITY_FURNITURE' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('CITY_FURNITURE','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD0_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD0_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD0_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD0_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD0_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD0_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD1_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD1_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD1_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD1_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD1_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD1_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD2_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD2_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD3_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD3_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD4_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD4_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='GENERIC_CITYOBJECT' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('GENERIC_CITYOBJECT','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='MASSPOINT_RELIEF' AND COLUMN_NAME='RELIEF_POINTS';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('MASSPOINT_RELIEF','RELIEF_POINTS',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='OPENING' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('OPENING','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='OPENING' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('OPENING','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='RELIEF_COMPONENT' AND COLUMN_NAME='EXTENT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('RELIEF_COMPONENT','EXTENT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD1_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD1_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD1_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD1_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SOLITARY_VEGETAT_OBJECT' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SOLITARY_VEGETAT_OBJECT','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SURFACE_DATA' AND COLUMN_NAME='GT_REFERENCE_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SURFACE_DATA','GT_REFERENCE_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SURFACE_GEOMETRY' AND COLUMN_NAME='SOLID_GEOMETRY';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SURFACE_GEOMETRY','SOLID_GEOMETRY',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='SURFACE_GEOMETRY' AND COLUMN_NAME='GEOMETRY';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('SURFACE_GEOMETRY','GEOMETRY',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TIN_RELIEF' AND COLUMN_NAME='BREAK_LINES';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TIN_RELIEF','BREAK_LINES',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TIN_RELIEF' AND COLUMN_NAME='CONTROL_POINTS';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TIN_RELIEF','CONTROL_POINTS',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TIN_RELIEF' AND COLUMN_NAME='STOP_LINES';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TIN_RELIEF','STOP_LINES',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TRANSPORTATION_COMPLEX' AND COLUMN_NAME='LOD0_NETWORK';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TRANSPORTATION_COMPLEX','LOD0_NETWORK',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD1_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD1_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD2_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD2_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD2_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD2_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD3_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD3_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD3_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD3_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD4_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD4_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL' AND COLUMN_NAME='LOD4_TERRAIN_INTERSECTION';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL','LOD4_TERRAIN_INTERSECTION',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_FURNITURE' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_FURNITURE','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_FURNITURE' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_FURNITURE','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_INSTALLATION' AND COLUMN_NAME='LOD2_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_INSTALLATION','LOD2_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_INSTALLATION' AND COLUMN_NAME='LOD2_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_INSTALLATION','LOD2_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_INSTALLATION' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_INSTALLATION','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_INSTALLATION' AND COLUMN_NAME='LOD3_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_INSTALLATION','LOD3_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_INSTALLATION' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_INSTALLATION','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_INSTALLATION' AND COLUMN_NAME='LOD4_OTHER_GEOM';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_INSTALLATION','LOD4_OTHER_GEOM',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_OPENING' AND COLUMN_NAME='LOD3_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_OPENING','LOD3_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='TUNNEL_OPENING' AND COLUMN_NAME='LOD4_IMPLICIT_REF_POINT';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('TUNNEL_OPENING','LOD4_IMPLICIT_REF_POINT',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='WATERBODY' AND COLUMN_NAME='LOD0_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('WATERBODY','LOD0_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

DELETE FROM USER_SDO_GEOM_METADATA WHERE TABLE_NAME='WATERBODY' AND COLUMN_NAME='LOD1_MULTI_CURVE';
INSERT INTO USER_SDO_GEOM_METADATA (TABLE_NAME, COLUMN_NAME, DIMINFO, SRID)
VALUES ('WATERBODY','LOD1_MULTI_CURVE',
MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X', 0.000, 10000000.000, 0.0005), MDSYS.SDO_DIM_ELEMENT('Y', 0.000, 10000000.000, 0.0005),MDSYS.SDO_DIM_ELEMENT('Z', -1000, 10000, 0.0005)), &SRSNO);

CREATE INDEX BREAKLINE_BREAK_SPX ON BREAKLINE_RELIEF (BREAK_LINES) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BREAKLINE_RIDGE_SPX ON BREAKLINE_RELIEF (RIDGE_OR_VALLEY_LINES) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD1TERR_SPX ON BRIDGE (LOD1_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD2CURVE_SPX ON BRIDGE (LOD2_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD2TERR_SPX ON BRIDGE (LOD2_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD3CURVE_SPX ON BRIDGE (LOD3_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD3TERR_SPX ON BRIDGE (LOD3_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD4CURVE_SPX ON BRIDGE (LOD4_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_LOD4TERR_SPX ON BRIDGE (LOD4_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONSTR_LOD1TERR_SPX ON BRIDGE_CONSTR_ELEMENT (LOD1_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONSTR_LOD2TERR_SPX ON BRIDGE_CONSTR_ELEMENT (LOD2_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONSTR_LOD3TERR_SPX ON BRIDGE_CONSTR_ELEMENT (LOD3_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONSTR_LOD4TERR_SPX ON BRIDGE_CONSTR_ELEMENT (LOD4_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD1REFPT_SPX ON BRIDGE_CONSTR_ELEMENT (LOD1_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD1XGEOM_SKX ON BRIDGE_CONSTR_ELEMENT (LOD1_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD2REFPT_SPX ON BRIDGE_CONSTR_ELEMENT (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD2XGEOM_SKX ON BRIDGE_CONSTR_ELEMENT (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD3REFPT_SPX ON BRIDGE_CONSTR_ELEMENT (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD3XGEOM_SKX ON BRIDGE_CONSTR_ELEMENT (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD4REFPT_SPX ON BRIDGE_CONSTR_ELEMENT (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_CONST_LOD4XGEOM_SKX ON BRIDGE_CONSTR_ELEMENT (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_FURN_LOD4REFPT_SPX ON BRIDGE_FURNITURE (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_FURN_LOD4XGEOM_SPX ON BRIDGE_FURNITURE (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_INST_LOD2REFPT_SPX ON BRIDGE_INSTALLATION (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_INST_LOD2XGEOM_SPX ON BRIDGE_INSTALLATION (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_INST_LOD3REFPT_SPX ON BRIDGE_INSTALLATION (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_INST_LOD3XGEOM_SPX ON BRIDGE_INSTALLATION (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_INST_LOD4REFPT_SPX ON BRIDGE_INSTALLATION (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_INST_LOD4XGEOM_SPX ON BRIDGE_INSTALLATION (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_OPEN_LOD3REFPT_SPX ON BRIDGE_OPENING (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BRIDGE_OPEN_LOD4REFPT_SPX ON BRIDGE_OPENING (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD1TERR_SPX ON BUILDING (LOD1_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD2CURVE_SPX ON BUILDING (LOD2_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD2TERR_SPX ON BUILDING (LOD2_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD3CURVE_SPX ON BUILDING (LOD3_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD3TERR_SPX ON BUILDING (LOD3_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD4CURVE_SPX ON BUILDING (LOD4_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BUILDING_LOD4TERR_SPX ON BUILDING (LOD4_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_FURN_LOD4REFPT_SPX ON BUILDING_FURNITURE (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_FURN_LOD4XGEOM_SPX ON BUILDING_FURNITURE (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_INST_LOD2REFPT_SPX ON BUILDING_INSTALLATION (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_INST_LOD2XGEOM_SPX ON BUILDING_INSTALLATION (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_INST_LOD3REFPT_SPX ON BUILDING_INSTALLATION (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_INST_LOD3XGEOM_SPX ON BUILDING_INSTALLATION (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_INST_LOD4REFPT_SPX ON BUILDING_INSTALLATION (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX BLDG_INST_LOD4XGEOM_SPX ON BUILDING_INSTALLATION (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITYMODEL_ENVELOPE_SPX ON CITYMODEL (ENVELOPE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITYOBJECT_ENVELOPE_SPX ON CITYOBJECT (ENVELOPE) INDEXTYPE IS MDSYS.SPATIAL_INDEX 
PARALLEL;

CREATE INDEX GROUP_XGEOM_SPX ON CITYOBJECTGROUP (OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD1REFPNT_SPX ON CITY_FURNITURE (LOD1_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD1TERR_SPX ON CITY_FURNITURE (LOD1_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD1XGEOM_SPX ON CITY_FURNITURE (LOD1_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD2REFPNT_SPX ON CITY_FURNITURE (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD2TERR_SPX ON CITY_FURNITURE (LOD2_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD2XGEOM_SPX ON CITY_FURNITURE (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD3REFPNT_SPX ON CITY_FURNITURE (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD3TERR_SPX ON CITY_FURNITURE (LOD3_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD3XGEOM_SPX ON CITY_FURNITURE (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD4REFPNT_SPX ON CITY_FURNITURE (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD4TERR_SPX ON CITY_FURNITURE (LOD4_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX CITY_FURN_LOD4XGEOM_SPX ON CITY_FURNITURE (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD0REFPNT_SPX ON GENERIC_CITYOBJECT (LOD0_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD0TERR_SPX ON GENERIC_CITYOBJECT (LOD0_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD0XGEOM_SPX ON GENERIC_CITYOBJECT (LOD0_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD1REFPNT_SPX ON GENERIC_CITYOBJECT (LOD1_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD1TERR_SPX ON GENERIC_CITYOBJECT (LOD1_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD1XGEOM_SPX ON GENERIC_CITYOBJECT (LOD1_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD2REFPNT_SPX ON GENERIC_CITYOBJECT (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD2TERR_SPX ON GENERIC_CITYOBJECT (LOD2_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD2XGEOM_SPX ON GENERIC_CITYOBJECT (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD3REFPNT_SPX ON GENERIC_CITYOBJECT (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD3TERR_SPX ON GENERIC_CITYOBJECT (LOD3_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD3XGEOM_SPX ON GENERIC_CITYOBJECT (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD4REFPNT_SPX ON GENERIC_CITYOBJECT (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD4TERR_SPX ON GENERIC_CITYOBJECT (LOD4_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX GEN_OBJECT_LOD4XGEOM_SPX ON GENERIC_CITYOBJECT (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX MASSPOINT_RELIEF_SPX ON MASSPOINT_RELIEF (RELIEF_POINTS) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX OPENING_LOD3REFPT_SPX ON OPENING (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX OPENING_LOD4REFPT_SPX ON OPENING (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX RELIEF_COMP_EXTENT_SPX ON RELIEF_COMPONENT (EXTENT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD1REFPT_SPX ON SOLITARY_VEGETAT_OBJECT (LOD1_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD1XGEOM_SPX ON SOLITARY_VEGETAT_OBJECT (LOD1_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD2REFPT_SPX ON SOLITARY_VEGETAT_OBJECT (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD2XGEOM_SPX ON SOLITARY_VEGETAT_OBJECT (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD3REFPT_SPX ON SOLITARY_VEGETAT_OBJECT (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD3XGEOM_SPX ON SOLITARY_VEGETAT_OBJECT (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD4REFPT_SPX ON SOLITARY_VEGETAT_OBJECT (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SOL_VEG_OBJ_LOD4XGEOM_SPX ON SOLITARY_VEGETAT_OBJECT (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SURFACE_DATA_SPX ON SURFACE_DATA (GT_REFERENCE_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX SURFACE_GEOM_SOLID_SPX ON SURFACE_GEOMETRY (SOLID_GEOMETRY) INDEXTYPE IS MDSYS.SPATIAL_INDEX 
PARAMETERS ('sdo_indx_dims=3') 
PARALLEL;

CREATE INDEX SURFACE_GEOM_SPX ON SURFACE_GEOMETRY (GEOMETRY) INDEXTYPE IS MDSYS.SPATIAL_INDEX 
PARALLEL;

CREATE INDEX TIN_RELF_BREAK_SPX ON TIN_RELIEF (BREAK_LINES) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TIN_RELF_CRTLPTS_SPX ON TIN_RELIEF (CONTROL_POINTS) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TIN_RELF_STOP_SPX ON TIN_RELIEF (STOP_LINES) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TRAN_COMPLEX_LOD0NET_SPX ON TRANSPORTATION_COMPLEX (LOD0_NETWORK) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD1TERR_SPX ON TUNNEL (LOD1_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD2CURVE_SPX ON TUNNEL (LOD2_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD2TERR_SPX ON TUNNEL (LOD2_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD3CURVE_SPX ON TUNNEL (LOD3_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD3TERR_SPX ON TUNNEL (LOD3_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD4CURVE_SPX ON TUNNEL (LOD4_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_LOD4TERR_SPX ON TUNNEL (LOD4_TERRAIN_INTERSECTION) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_FURN_LOD4REFPT_SPX ON TUNNEL_FURNITURE (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_FURN_LOD4XGEOM_SPX ON TUNNEL_FURNITURE (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_INST_LOD2REFPT_SPX ON TUNNEL_INSTALLATION (LOD2_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_INST_LOD2XGEOM_SPX ON TUNNEL_INSTALLATION (LOD2_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_INST_LOD3REFPT_SPX ON TUNNEL_INSTALLATION (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_INST_LOD3XGEOM_SPX ON TUNNEL_INSTALLATION (LOD3_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_INST_LOD4REFPT_SPX ON TUNNEL_INSTALLATION (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_INST_LOD4XGEOM_SPX ON TUNNEL_INSTALLATION (LOD4_OTHER_GEOM) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_OPEN_LOD3REFPT_SPX ON TUNNEL_OPENING (LOD3_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX TUNNEL_OPEN_LOD4REFPT_SPX ON TUNNEL_OPENING (LOD4_IMPLICIT_REF_POINT) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX WATERBODY_LOD0CURVE_SPX ON WATERBODY (LOD0_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

CREATE INDEX WATERBODY_LOD1CURVE_SPX ON WATERBODY (LOD1_MULTI_CURVE) INDEXTYPE IS MDSYS.SPATIAL_INDEX;

